--[[
==============================================================================
Prospector 3 - Plugin for Vendetta Online (all platforms)
==============================================================================
Originally created by PaKettle, 13 August 2016 to 11 August 2020
This version largely rewritten by Snib#9290, June-August 2022

Fully configurable via GUI.

User commands:
 /pr /prospector             - show GUI
 /rs /roidscan               - toggle automatic asteroid scanner

 /prForceDatabaseUpgrade     - force a database upgrade

 /prImportFromTargetless     - import new ore data from targetless
 /prExportToTargetless       - export ore data to targetless
 /prImportFromRoidDB         - import new ore data from MDI RoidDB
==============================================================================
]]

declare("PR")
PR = {
    Version = "3.4.2~Snib",
    PluginTitle = "", -- autogenerateed from version
    Sector = 0, -- current sector id
    LoadedSector = 0, -- id of currently loaded sector
    PluginReady = false, -- true after PR.Init() (and PR.RegisterEvents) ran
    SystemNotesPrefix = 91000000, -- prefix for notes stored by this plugin
    GUI = {
        Color = {}, -- presets for iup fgcolor/bgcolor
        Views = { "Universe", "Sector", "Search", "Config" },
        VerbosityLevels = { "Quiet", "Normal", "Noisy" },
        SectorSortOptions = { "ById", "ByXPV", "ByMass", "ByValue", "ByOre" },
        Areas = { "Gray", "Itani", "Serco", "UIT" },
        EditableValues = { "StartupDelay", "AutoScanFrequency", "MinXPV" },
        EditableValuesUnit = { "ms", "ms", "XPV" },
        SectorRoidList = {}, -- caches asteroid list while GUI is open
        SystemData = {}, -- caches data for all loaded systems while GUI is open
        CurrentSectorShowing = false, -- if GUI is open showing the current sector
        DisplayedSector = 0, -- id of sector shown by sector view
        ListedItems = {}, -- ids of displayed asteroids
        MaxDisplayRows = 0, -- length of displayed list
        View = "Sector", -- currently active view
        SectorOres = {}, -- list of ores in currently displayed sector
        OreSort = 0, -- id of the ore in SectorOres that ByOre is sorting by
        ScanTop30Idx = false, -- index for the top30 search. disabled if false
    },
    AutoScanActive = false,
    AutoScanTimer = Timer(),
    SingleSearchTarget = 0, -- single search mode if this is set to an asteroid id
    SingleSearchTimer = Timer(),
    SpamHold = 0,
    SpamTimer = Timer(),
    LastDisplayedRoidId = 0, -- keep track of last displayed roid so we don't repeat it
    RoidNode = 0, -- radar node for asteroids in the current sector
    MiningBeamEquipped = false,
    ScannerEquipped = false,
    ScanRange = 0, -- mineral scanner range (maximum of built-in and addon scanners)
    MassTarget = 0, -- mass test target asteroid id
    MassTestActive = false,
    MassScanCount = 0, -- mass test scan count
    MassScanMax = 0, -- max mass test scan count, measurement when reached
    LastTemp = 0, -- last measured asteroid temperature
    MiningBeamHeatRate = 0, -- average heat rate of all equipped mining beams
    MiningBeamCount = 0, -- number of equipped mining beams
    SaveSectorId = 0, -- id of sector to be saved
    SaveSectorTimer = Timer(),
    SystemUpdateSectorId = 0, -- system data update pending for this sector
    SystemUpdateTimer = Timer(),
    Config = {}, -- user configuration
    Data = {}, -- data for currently loaded sector
    Color = {}, -- text color presets
    Ore = {},
    SystemOwnership = {},
    Interop = { -- interoperability  with other plugins
        IsBotwatcherInstalled = false, -- whether the Botwatcher plugin is loaded
        IsTargetlessInstalled = false, -- whether the targetless plugin is loaded
    },
}
PR.PluginTitle = "Prospector v" .. PR.Version

dofile("static_data.lua")

PR.Config = {
    Version = PR.Version, -- Version string determines potential upgrade process
    AutoScanFrequency = 1000, -- frequency in ms for auto scanner to retarget
    StartupDelay = 5000, -- delay before registering most event handlers
    MinXPV = 0, -- asteroids with XPV below this will be shown as "junk"
    MaxIdDigits = 2, -- minimum digits for roid id leading 0 formatting
    SectorSort = "ById", -- GUI sector sorting
    UniverseSort = "ByOre", -- GUI Universe sorting
    Search = { -- search settings
        Ore = PR.Ore[1].shortname,
        Area = PR.SystemOwnership[1],
        System = 1
    },
    COLOR = true, -- status message output in color or white only
    VerbosityLevel = 3, -- controls which messages are printed and how often
    PrintToHUD = true -- controls whether some messages get printed on HUD
}

-- generate settings for ore search threshold percentages
for i = 1, PR.Ore.n do
    local param = "SearchPercent_" .. PR.Ore[i].shortname
    table.insert(PR.GUI.EditableValues, param)
    table.insert(PR.GUI.EditableValuesUnit, "%")
    PR.Config[param] = 0
end

dofile("utils.lua")
dofile("equipment.lua")
dofile("reminder.lua")
dofile("roid.lua")
dofile("roid_display.lua")
dofile("scanners.lua")
dofile("scan_event.lua")
dofile("sector.lua")
dofile("system.lua")
dofile("upgrade.lua")
dofile("GUI/main.lua")
dofile("interop/main.lua")
dofile("interop/targetless.lua")
dofile("interop/mdi_roiddb.lua")

function PR.Init()
    UnregisterEvent(PR.Init, "ENTERED_STATION")
    UnregisterEvent(PR.Init, "SHIP_SPAWNED")

    -- config needs to be loaded first
    local config = unspickle(LoadSystemNotes(PR.SystemNotesPrefix)) or {}
    for key, val in pairs(config) do
        PR.Config[key] = val
    end

    -- do any plugin version upgrade before loading any further data
    if PR.Version ~= PR.Config.Version then
        PR.UpgradeVersion()
    end

    -- load ship and sector data
    PR.CheckEquipment()
    PR.SectorChanged()

    -- Workaround for "invalid key to 'next'" crash when targetless is also
    -- installed, which seems to be a bug on the game's side iteration over
    -- event handlers
    local delay = Timer()
    return delay:SetTimeout(PR.Config.StartupDelay, PR.RegisterEvents)
end

function PR.RegisterEvents()
    RegisterEvent(PR.TargetScanned, "TARGET_SCANNED")
    RegisterEvent(PR.FreshStart, "ENTERING_STATION")
    RegisterEvent(PR.SectorChanged, "SECTOR_CHANGED")
    RegisterEvent(PR.CheckEquipment, "SHIP_UPDATED")
    RegisterEvent(PR.CheckEquipment, "SHIP_SPAWNED")
    RegisterEvent(PR.CheckEquipment, "SHIP_CHANGED")
    RegisterEvent(PR.Interop.ReleaseTargetlessLock, "PLAYER_LOGGED_OUT")
    RegisterEvent(PR.SaveNow, "PLAYER_LOGGED_OUT")
    RegisterEvent(PR.SaveNow, "UNLOAD_INTERFACE")

    PR.PluginReady = true
    return PR.Print(PR.PluginTitle .. " ready", true)
end

-- init as soon as we can check equipment
RegisterEvent(PR.Init, "ENTERED_STATION")
RegisterEvent(PR.Init, "SHIP_SPAWNED")

---------------------------
-- SLASH COMMANDS
---------------------------
RegisterUserCommand("roidscan", PR.AutoScanToggle)
RegisterUserCommand("rs", PR.AutoScanToggle)

RegisterUserCommand("prospector", PR.GUI.ShowGUI)
RegisterUserCommand("pr", PR.GUI.ShowGUI)
RegisterUserCommand("ore", PR.GUI.ShowGUI) -- deprecated

RegisterUserCommand("prForceDatabaseUpgrade", PR.ForceUpgradeVersion)

RegisterUserCommand("prImportFromTargetless", PR.Interop.ImportFromTargetless)
RegisterUserCommand("prExportToTargetless", PR.Interop.ExportToTargetless)
RegisterUserCommand("prImportFromRoidDB", PR.Interop.ImportFromRoidDB)
