-- load all system data indexed by system id into allSystemData,
-- but only for systems that have not been loaded yet
function PR.LoadAllSystemData(allSystemData)
    for sysid = 1, PR.SystemOwnership.n do
        PR.LoadSingleSystemData(allSystemData, sysid)
    end
end

-- load a single system's data indexed by system id into allSystemData,
-- but only for systems that have not been loaded yet
function PR.LoadSingleSystemData(allSystemData, sysid)
    if not allSystemData[sysid] then
        local systemname = PR.GetSystemName(sysid)
        if systemname then
            allSystemData[sysid] = PR.LoadSystemData(sysid)
            allSystemData[sysid].systemname = systemname
        else
            allSystemData[sysid] = false
        end
    end
end

-- load and return system notes
function PR.LoadSystemData(systemId)
    return unspickle(LoadSystemNotes(PR.SystemNotesPrefix + 10000 + systemId)) or {}
end

-- save system notes
function PR.SaveSystemData(systemId, systemData)
    return SaveSystemNotes(spickle(systemData), PR.SystemNotesPrefix + 10000 + systemId)
end

-- writes new asteroid ore counts for given sector to SystemData
function PR.UpdateSystemData(sectorId, sectorData, systemData)
    if not sectorData then return end

    -- get asteroid count for each ore in sector
    local sector_ores = {}
    for _, roid in pairs(sectorData) do
        for i = 1, PR.Ore.n do
            local ore = PR.Ore[i].shortname
            if roid[ore] then
                local percent = roid[ore]
                sector_ores[ore] = sector_ores[ore] or {}
                sector_ores[ore].all = (sector_ores[ore].all or 0) + 1
                if PR.Config["SearchPercent_" .. ore] > 0 and
                    percent >= PR.Config["SearchPercent_" .. ore]
                then
                    sector_ores[ore].threshold = (sector_ores[ore].threshold or 0) + 1
                end
                if (sector_ores[ore].max or 0) < percent then
                    sector_ores[ore].max = percent
                end
            end
        end
    end

    -- update sector data in system ore summary table
    for i = 1, PR.Ore.n do
        local ore = PR.Ore[i].shortname
        local oretable = systemData[ore] or {}
        if sector_ores[ore] then
            oretable[sectorId] = {
                sector_ores[ore].all,
                sector_ores[ore].max,
                sector_ores[ore].threshold, -- this can be nil
            }
        end
        systemData[ore] = oretable
    end
end

-- recreates ore count summaries for all systems
function PR.RecreateAllSystemData()
    for systemId = 1, PR.SystemOwnership.n do
        local systemData = {}
        for sectorId = systemId * 256 - 255, systemId * 256 do
            local sectorData = PR.LoadSectorData(sectorId)
            PR.UpdateSystemData(sectorId, sectorData.RoidList, systemData)
        end
        PR.SaveSystemData(systemId, systemData)
    end
end

-- loads system data for system that PR.SystemUpdateSectorId is in, then
-- updates with sector data and saves it all
function PR.SystemDataUpdateNow()
    if PR.SystemUpdateTimer:IsActive() then PR.SystemUpdateTimer:Kill() end
    if PR.SystemUpdateSectorId > 0 then
        PR.LoadSector(PR.SystemUpdateSectorId)
        local systemId = GetSystemID(PR.SystemUpdateSectorId)
        local systemData = PR.LoadSystemData(systemId)
        PR.UpdateSystemData(PR.LoadedSector, PR.Data.RoidList, systemData)
        PR.SaveSystemData(systemId, systemData)
        PR.SystemUpdateSectorId = 0
    end
end

-- starts/resets timer for PR.SystemDataUpdateNow() to update asteroid
-- ore counts in sector to system summary data
function PR.SystemDataUpdate(SectorId)
    -- if we have a pending update from another sector force an update
    -- - this situation should never occur
    if PR.SystemUpdateSectorId > 0 and PR.SystemUpdateSectorId ~= SectorId then
        PR.SystemDataUpdateNow()
    end
    PR.SystemUpdateSectorId = SectorId
    return PR.SystemUpdateTimer:SetTimeout(70000, PR.SystemDataUpdateNow)
end

-- checks system data for a specific ore and returns asteroid count
-- @oreData must be system summary data for said ore
function PR.GetOreCountInSystem(oreData)
    local orecount = 0
    for _, data in pairs(oreData) do
        orecount = orecount + data[1]
    end
    return orecount
end

-- returns known system name or nil
function PR.GetSystemName(sysid)
    if sysid == 33 then return "Unknown" end
    return SystemNames[sysid]
end

-- fill PR.Systems
for i = 1, PR.SystemOwnership.n do
    local systemname = PR.GetSystemName(i)
    if systemname then table.insert(PR.Systems, { i, systemname }) end
end
table.sort(PR.Systems, function(a, b) return a[2] < b[2] end)
PR.Systems.n = #PR.Systems
